#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_bgp_neighbor_address_family
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_bgp_neighbor_address_family
short_description: BGP Neighbor Address Family resource module.
description:
- This module manages BGP Neighbor Address Family configuration on devices running Cisco NX-OS.
version_added: 2.0.0
notes:
- Tested against NX-OS 9.3.6.
- Unsupported for Cisco MDS
- For managing BGP address family configurations please use
  the M(cisco.nxos.nxos_bgp_address_family) module.
- This module works with connection C(network_cli) and C(httpapi).
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section '^router bgp').
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: BGP Neighbor AF configuration.
    type: dict
    suboptions:
      as_number:
        description: Autonomous System Number of the router.
        type: str
      neighbors: &nbr
        description: A list of BGP Neighbor AF configuration.
        type: list
        elements: dict
        suboptions:
          neighbor_address:
            description: IP/IPv6 address of the neighbor.
            type: str
            required: true
          address_family:
            description: BGP Neighbor Address Family related configurations.
            type: list
            elements: dict
            suboptions:
              afi:
                description: Address Family indicator.
                type: str
                choices: ["ipv4", "ipv6", "link-state", "vpnv4", "vpnv6", "l2vpn"]
                required: true
              safi:
                description: Sub Address Family indicator.
                type: str
                choices: ["unicast", "multicast", "mvpn", "evpn"]
              advertise_map:
                description: Specify route-map for conditional advertisement.
                type: dict
                suboptions:
                  route_map:
                    description: Route-map name.
                    type: str
                    required: true
                  exist_map:
                    description: Condition route-map to advertise only when prefix in condition exists.
                    type: str
                  non_exist_map:
                    description: Condition route-map to advertise only when prefix in condition does not exist.
                    type: str
              advertisement_interval:
                description: Minimum interval between sending BGP routing updates.
                type: int
              allowas_in:
                description: Accept as-path with my AS present in it.
                type: dict
                suboptions:
                  set:
                    description: Activate allowas-in property.
                    type: bool
                  max_occurences:
                    description: Number of occurrences of AS number, default is 3.
                    type: int
              as_override:
                description: Override matching AS-number while sending update.
                type: bool
              capability:
                description: Advertise capability to the peer.
                type: dict
                suboptions:
                  additional_paths:
                    description: Additional paths capability.
                    type: dict
                    suboptions:
                      receive:
                        description: Additional paths Receive capability.
                        type: str
                        choices: ["enable", "disable"]
                      send:
                        description: Additional paths Send capability.
                        type: str
                        choices: ["enable", "disable"]
              default_originate:
                description: Originate a default toward this peer.
                type: dict
                suboptions:
                  set:
                    description: Set default-originate attribute.
                    type: bool
                  route_map:
                    description: Route-map to specify criteria for originating default.
                    type: str
              disable_peer_as_check:
                description: Disable checking of peer AS-number while advertising.
                type: bool
              filter_list:
                description: Name of filter-list.
                type: dict
                suboptions:
                  inbound:
                    description: Apply policy to incoming routes.
                    type: str
                  outbound:
                    description: Apply policy to outgoing routes.
                    type: str
              inherit:
                description: Inherit a template.
                type: dict
                suboptions:
                  template:
                    description: Template name.
                    type: str
                  sequence:
                    description: Sequence number.
                    type: int
              maximum_prefix:
                description: Maximum number of prefixes from this neighbor.
                type: dict
                suboptions:
                  max_prefix_limit:
                    description: Maximum prefix limit.
                    type: int
                  generate_warning_threshold:
                    description: Threshold percentage at which to generate a warning.
                    type: int
                  restart_interval:
                    description: Restart bgp connection after limit is exceeded.
                    type: int
                  warning_only:
                    description: Only give a warning message when limit is exceeded.
                    type: bool
              next_hop_self:
                description: Set our address as nexthop (non-reflected).
                type: dict
                suboptions:
                  set:
                    description: Set next-hop-self attribute.
                    type: bool
                  all_routes:
                    description: Set our address as nexthop for all routes.
                    type: bool
              next_hop_third_party:
                description: Compute a third-party nexthop if possible.
                type: bool
              prefix_list:
                description: Apply prefix-list.
                type: dict
                suboptions:
                  inbound:
                    description: Apply policy to incoming routes.
                    type: str
                  outbound:
                    description: Apply policy to outgoing routes.
                    type: str
              rewrite_evpn_rt_asn:
                description: Auto generate RTs for EBGP neighbor.
                type: bool
              rewrite_rt_asn:
                description: Auto generate RTs for EBGP neighbor.
                type: bool
              route_map:
                description: Apply route-map to neighbor.
                type: dict
                suboptions:
                  inbound:
                    description: Apply policy to incoming routes.
                    type: str
                  outbound:
                    description: Apply policy to outgoing routes.
                    type: str
              route_reflector_client:
                description: Configure a neighbor as Route reflector client.
                type: bool
              send_community:
                description: Send Community attribute to this neighbor.
                type: dict
                suboptions:
                  set:
                    description: Set send-community attribute.
                    type: bool
                  extended:
                    description: Send Extended Community attribute.
                    type: bool
                  standard:
                    description: Send Standard Community attribute.
                    type: bool
                  both:
                    description: Send Standard and Extended Community attributes.
                    type: bool
              soft_reconfiguration_inbound:
                description: Soft reconfiguration.
                type: dict
                suboptions:
                  set:
                    description: Set soft-reconfiguration inbound attribute.
                    type: bool
                  always:
                    description: Always perform inbound soft reconfiguration.
                    type: bool
              soo:
                description: Specify Site-of-origin extcommunity.
                type: str
              suppress_inactive:
                description: Advertise only active routes to peer.
                type: bool
              unsuppress_map:
                description: Route-map to selectively unsuppress suppressed routes.
                type: str
              weight:
                description: Set default weight for routes from this neighbor.
                type: int
      vrfs:
        description: Virtual Router Context.
        type: list
        elements: dict
        suboptions:
          vrf:
            description: VRF name.
            type: str
          neighbors: *nbr
  state:
    description:
    - The state the configuration should be left in.
    - State I(deleted) only removes BGP attributes that this modules
      manages and does not negate the BGP process completely.
    - Refer to examples for more details.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - gathered
    - rendered
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# Nexus9000v#

- name: Merge the provided configuration with the existing running configuration
  cisco.nxos.nxos_bgp_neighbor_address_family: &id001
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.32
          address_family:
            - afi: ipv4
              safi: unicast
              maximum_prefix:
                max_prefix_limit: 20
                generate_warning_threshold: 75
              weight: 100
              prefix_list:
                inbound: rmap1
                outbound: rmap2
            - afi: ipv6
              safi: unicast
        - neighbor_address: 192.0.2.33
          address_family:
            - afi: ipv4
              safi: multicast
              inherit:
                template: BasePolicy
                sequence: 200
      vrfs:
        - vrf: site-1
          neighbors:
            - neighbor_address: 203.0.113.1
              address_family:
                - afi: ipv4
                  safi: unicast
                  suppress_inactive: true
                  next_hop_self:
                    set: true
            - neighbor_address: 203.0.113.2
              address_family:
                - afi: ipv6
                  safi: unicast
                - afi: ipv4
                  safi: multicast
                  send_community:
                    set: true

# Task output:
# ------------
#  before: {}
#
#  commands:
#  - router bgp 65536
#  - neighbor 192.0.2.32
#  - address-family ipv4 unicast
#  - maximum-prefix 20 75
#  - weight 100
#  - prefix-list rmap1 in
#  - prefix-list rmap2 out
#  - address-family ipv6 unicast
#  - neighbor 192.0.2.33
#  - address-family ipv4 multicast
#  - inherit peer-policy BasePolicy 200
#  - vrf site-1
#  - neighbor 203.0.113.1
#  - address-family ipv4 unicast
#  - suppress-inactive
#  - next-hop-self
#  - neighbor 203.0.113.2
#  - address-family ipv6 unicast
#  - address-family ipv4 multicast
#  - send-community
#
#  after:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#            maximum_prefix:
#              max_prefix_limit: 20
#              generate_warning_threshold: 75
#            weight: 100
#            prefix_list:
#              inbound: rmap1
#              outbound: rmap2
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       maximum-prefix 20 75
#       weight 100
#       prefix-list rmap1 in
#       prefix-list rmap2 out
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast
#

# Using replaced

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       maximum-prefix 20 75
#       weight 100
#       prefix-list rmap1 in
#       prefix-list rmap2 out
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast
#

- name: Replace specified neighbor AFs with given configuration
  cisco.nxos.nxos_bgp_neighbor_address_family: &replaced
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.32
          address_family:
            - afi: ipv4
              safi: unicast
              weight: 110
            - afi: ipv6
              safi: unicast
        - neighbor_address: 192.0.2.33
          address_family:
            - afi: ipv4
              safi: multicast
              inherit:
                template: BasePolicy
                sequence: 200
      vrfs:
        - vrf: site-1
          neighbors:
            - neighbor_address: 203.0.113.1
              address_family:
                - afi: ipv4
                  safi: unicast
            - neighbor_address: 203.0.113.2
              address_family:
                - afi: ipv6
                  safi: unicast
                - afi: ipv4
                  safi: multicast
                  send_community:
                    set: true
    state: replaced

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#            maximum_prefix:
#              max_prefix_limit: 20
#              generate_warning_threshold: 75
#            weight: 100
#            prefix_list:
#              inbound: rmap1
#              outbound: rmap2
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast
#
#  commands:
#    - router bgp 65536
#    - neighbor 192.0.2.32
#    - address-family ipv4 unicast
#    - no maximum-prefix 20 75
#    - weight 110
#    - no prefix-list rmap1 in
#    - no prefix-list rmap2 out
#    - vrf site-1
#    - neighbor 203.0.113.1
#    - address-family ipv4 unicast
#    - no suppress-inactive
#    - no next-hop-self
#
#  after:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#            weight: 110
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       weight 110
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast
#

# Using overridden

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       maximum-prefix 20 75
#       weight 100
#       prefix-list rmap1 in
#       prefix-list rmap2 out
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast
#

- name: Override all BGP AF configuration with provided configuration
  cisco.nxos.nxos_bgp_neighbor_address_family:
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.32
          address_family:
            - afi: ipv4
              safi: unicast
      vrfs:
        - vrf: site-1
          neighbors:
            - neighbor_address: 203.0.113.1
              address_family:
                - afi: ipv4
                  safi: unicast
                  suppress_inactive: true
                  next_hop_self:
                    set: true
    state: overridden

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#            maximum_prefix:
#              max_prefix_limit: 20
#              generate_warning_threshold: 75
#            weight: 100
#            prefix_list:
#              inbound: rmap1
#              outbound: rmap2
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast
#
#  commands:
#    - router bgp 65536
#    - neighbor 192.0.2.32
#    - address-family ipv4 unicast
#    - no maximum-prefix 20 75
#    - no weight 100
#    - no prefix-list rmap1 in
#    - no prefix-list rmap2 out
#    - no address-family ipv6 unicast
#    - neighbor 192.0.2.33
#    - no address-family ipv4 multicast
#    - vrf site-1
#    - neighbor 203.0.113.2
#    - no address-family ipv4 multicast
#    - no address-family ipv6 unicast
#
#  after:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self

# Using deleted to remove specified neighbor AFs

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       maximum-prefix 20 75
#       weight 100
#       prefix-list rmap1 in
#       prefix-list rmap2 out
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast
#

- name: Delete BGP configs handled by this module
  cisco.nxos.nxos_bgp_neighbor_address_family:
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.32
          address_family:
            - afi: ipv4
              safi: unicast
      vrfs:
        - vrf: site-1
          neighbors:
            - neighbor_address: 203.0.113.2
              address_family:
                - afi: ipv6
                  safi: unicast
    state: deleted

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#            maximum_prefix:
#              max_prefix_limit: 20
#              generate_warning_threshold: 75
#            weight: 100
#            prefix_list:
#              inbound: rmap1
#              outbound: rmap2
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast
#
#  commands:
#    - router bgp 65536
#    - neighbor 192.0.2.32
#    - no address-family ipv4 unicast
#    - vrf site-1
#    - neighbor 203.0.113.2
#    - no address-family ipv6 unicast
#
#  after:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#
# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#

# Using deleted to remove all neighbor AFs

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       maximum-prefix 20 75
#       weight 100
#       prefix-list rmap1 in
#       prefix-list rmap2 out
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast
#

- name: Delete all BGP neighbor AF configs handled by this module
  cisco.nxos.nxos_bgp_neighbor_address_family:
    state: deleted

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#            safi: unicast
#            maximum_prefix:
#              max_prefix_limit: 20
#              generate_warning_threshold: 75
#            weight: 100
#            prefix_list:
#              inbound: rmap1
#              outbound: rmap2
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast
#
#  commands:
#    - router bgp 65536
#    - neighbor 192.0.2.32
#    - no address-family ipv4 unicast
#    - no address-family ipv6 unicast
#    - neighbor 192.0.2.33
#    - no address-family ipv4 multicast
#    - vrf site-1
#    - neighbor 203.0.113.1
#    - no address-family ipv4 unicast
#    - neighbor 203.0.113.2
#    - no address-family ipv6 unicast
#    - no address-family ipv4 multicast
#
#  after:
#    as_number: "65536"
#
# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   neighbor 192.0.2.32
#   neighbor 192.0.2.33
#   vrf site-1
#     neighbor 203.0.113.1
#     neighbor 203.0.113.2
#

# Using rendered

- name: Render platform specific configuration lines with state rendered (without connecting to the device)
  cisco.nxos.nxos_bgp_neighbor_address_family:
    config:
      as_number: 65536
      neighbors:
        - neighbor_address: 192.0.2.32
          address_family:
            - afi: ipv4
              safi: unicast
              maximum_prefix:
                max_prefix_limit: 20
                generate_warning_threshold: 75
              weight: 100
              prefix_list:
                inbound: rmap1
                outbound: rmap2
            - afi: ipv6
              safi: unicast
        - neighbor_address: 192.0.2.33
          address_family:
            - afi: ipv4
              safi: multicast
              inherit:
                template: BasePolicy
                sequence: 200
      vrfs:
        - vrf: site-1
          neighbors:
            - neighbor_address: 203.0.113.1
              address_family:
                - afi: ipv4
                  safi: unicast
                  suppress_inactive: true
                  next_hop_self:
                    set: true
            - neighbor_address: 203.0.113.2
              address_family:
                - afi: ipv6
                  safi: unicast
                - afi: ipv4
                  safi: multicast
                  send_community:
                    set: true
    state: rendered

# Task output:
# ------------
#  rendered:
#    - router bgp 65536
#    - neighbor 192.0.2.32
#    - address-family ipv4 unicast
#    - maximum-prefix 20 75
#    - weight 100
#    - prefix-list rmap1 in
#    - prefix-list rmap2 out
#    - address-family ipv6 unicast
#    - neighbor 192.0.2.33
#    - address-family ipv4 multicast
#    - inherit peer-policy BasePolicy 200
#    - vrf site-1
#    - neighbor 203.0.113.1
#    - address-family ipv4 unicast
#    - suppress-inactive
#    - next-hop-self
#    - neighbor 203.0.113.2
#    - address-family ipv6 unicast
#    - address-family ipv4 multicast
#    - send-community

# Using parsed

# parsed.cfg
# ------------
# router bgp 65536
#   neighbor 192.0.2.32
#     address-family ipv4 unicast
#       maximum-prefix 20 75
#       weight 100
#       prefix-list rmap1 in
#       prefix-list rmap2 out
#     address-family ipv6 unicast
#   neighbor 192.0.2.33
#     address-family ipv4 multicast
#       inherit peer-policy BasePolicy 200
#   vrf site-1
#     neighbor 203.0.113.1
#       address-family ipv4 unicast
#         suppress-inactive
#         next-hop-self
#     neighbor 203.0.113.2
#       address-family ipv4 multicast
#         send-community
#       address-family ipv6 unicast

- name: Parse externally provided BGP neighbor AF config
  register: result
  cisco.nxos.nxos_bgp_neighbor_address_family:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output:
# ------------
#  parsed:
#    as_number: "65536"
#    neighbors:
#      - neighbor_address: 192.0.2.32
#        address_family:
#          - afi: ipv4
#           safi: unicast
#           maximum_prefix:
#              max_prefix_limit: 20
#              generate_warning_threshold: 75
#           weight: 100
#            prefix_list:
#              inbound: rmap1
#              outbound: rmap2
#          - afi: ipv6
#            safi: unicast
#      - neighbor_address: 192.0.2.33
#        address_family:
#          - afi: ipv4
#            safi: multicast
#            inherit:
#              template: BasePolicy
#              sequence: 200
#    vrfs:
#      - vrf: site-1
#        neighbors:
#          - neighbor_address: 203.0.113.1
#            address_family:
#              - afi: ipv4
#                safi: unicast
#                suppress_inactive: true
#                next_hop_self:
#                  set: true
#          - neighbor_address: 203.0.113.2
#            address_family:
#              - afi: ipv4
#                safi: multicast
#                send_community:
#                  set: true
#              - afi: ipv6
#                safi: unicast
"""
RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - router bgp 65536
    - neighbor 192.0.2.32
    - address-family ipv4 unicast
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - router bgp 65536
    - neighbor 192.0.2.32
    - address-family ipv4 unicast
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.bgp_neighbor_address_family.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.bgp_neighbor_address_family.bgp_neighbor_address_family import (
    Bgp_neighbor_address_family,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Bgp_neighbor_address_familyArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Bgp_neighbor_address_family(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2013, Jan-Piet Mens <jpmens(at)gmail.com>
# (m) 2016, Mihai Moldovanu <mihaim@tfm.ro>
# (m) 2017, Juan Manuel Parrilla <jparrill@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Jan-Piet Mens (@jpmens)
name: etcd
short_description: Get info from an etcd server
description:
  - Retrieves data from an etcd server.
options:
  _terms:
    description:
      - The list of keys to lookup on the etcd server.
    type: list
    elements: string
    required: true
  url:
    description:
      - Environment variable with the URL for the etcd server.
    type: string
    default: 'http://127.0.0.1:4001'
    env:
      - name: ANSIBLE_ETCD_URL
  version:
    description:
      - Environment variable with the etcd protocol version.
    type: string
    default: 'v1'
    env:
      - name: ANSIBLE_ETCD_VERSION
  validate_certs:
    description:
      - Toggle checking that the ssl certificates are valid, you normally only want to turn this off with self-signed certs.
    default: true
    type: boolean
seealso:
  - module: community.general.etcd3
  - plugin: community.general.etcd3
    plugin_type: lookup
"""

EXAMPLES = r"""
- name: "a value from a locally running etcd"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd', 'foo/bar') }}"

- name: "values from multiple folders on a locally running etcd"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd', 'foo', 'bar', 'baz') }}"

- name: "you can set server options inline"
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.etcd', 'foo', version='v2', url='http://192.168.0.27:4001') }}"
"""

RETURN = r"""
_raw:
  description:
    - List of values associated with input keys.
  type: list
  elements: string
"""

import json

from ansible.module_utils.urls import open_url
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

# this can be made configurable, not should not use ansible.cfg
#
# Made module configurable from playbooks:
# If etcd  v2 running on host 192.168.1.21 on port 2379
# we can use the following in a playbook to retrieve /tfm/network/config key
#
# - ansible.builtin.debug: msg={{lookup('etcd','/tfm/network/config', url='http://192.168.1.21:2379' , version='v2')}}
#
# Example Output:
#
# TASK [debug] *******************************************************************
# ok: [localhost] => {
#     "msg": {
#         "Backend": {
#             "Type": "vxlan"
#         },
#         "Network": "172.30.0.0/16",
#         "SubnetLen": 24
#     }
# }
#
#
#
#


class Etcd:
    def __init__(self, url, version, validate_certs):
        self.url = url
        self.version = version
        self.baseurl = f"{self.url}/{self.version}/keys"
        self.validate_certs = validate_certs

    def _parse_node(self, node):
        # This function will receive all etcd tree,
        # if the level requested has any node, the recursion starts
        # create a list in the dir variable and it is passed to the
        # recursive function, and so on, if we get a variable,
        # the function will create a key-value at this level and
        # undoing the loop.
        path = {}
        if node.get("dir", False):
            for n in node.get("nodes", []):
                path[n["key"].split("/")[-1]] = self._parse_node(n)

        else:
            path = node["value"]

        return path

    def get(self, key):
        url = f"{self.baseurl}/{key}?recursive=true"
        data = None
        value = {}
        try:
            r = open_url(url, validate_certs=self.validate_certs)
            data = r.read()
        except Exception:
            return None

        try:
            # I will not support Version 1 of etcd for folder parsing
            item = json.loads(data)
            if self.version == "v1":
                # When ETCD are working with just v1
                if "value" in item:
                    value = item["value"]
            else:
                if "node" in item:
                    # When a usual result from ETCD
                    value = self._parse_node(item["node"])

            if "errorCode" in item:
                # Here return an error when an unknown entry responds
                value = "ENOENT"
        except Exception:
            raise

        return value


class LookupModule(LookupBase):
    def run(self, terms, variables, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        validate_certs = self.get_option("validate_certs")
        url = self.get_option("url")
        version = self.get_option("version")

        etcd = Etcd(url=url, version=version, validate_certs=validate_certs)

        ret = []
        for term in terms:
            key = term.split()[0]
            value = etcd.get(key)
            ret.append(value)
        return ret



#!/usr/bin/python

# Copyright (c) 2017, Eike Frost <ei@kefro.st>
# Copyright (c) 2021, Christophe Gilles <christophe.gilles54@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or
# https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_realm_key

short_description: Allows administration of Keycloak realm keys using Keycloak API

version_added: 7.5.0

description:
  - This module allows the administration of Keycloak realm keys using the Keycloak REST API. It requires access to the REST
    API using OpenID Connect; the user connecting and the realm being used must have the requisite access rights. In a default
    Keycloak installation, admin-cli and an admin user would work, as would a separate realm definition with the scope tailored
    to your needs and a user having the expected roles.
  - The names of module options are snake_cased versions of the camelCase ones found in the Keycloak API and its documentation
    at U(https://www.keycloak.org/docs-api/8.0/rest-api/index.html). Aliases are provided so camelCased versions can be used
    as well.
  - This module is unable to detect changes to the actual cryptographic key after importing it. However, if some other property
    is changed alongside the cryptographic key, then the key also changes as a side-effect, as the JSON payload needs to include
    the private key. This can be considered either a bug or a feature, as the alternative would be to always update the realm
    key whether it has changed or not.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: partial
  action_group:
    version_added: 10.2.0

options:
  state:
    description:
      - State of the keycloak realm key.
      - On V(present), the realm key is created (or updated if it exists already).
      - On V(absent), the realm key is removed if it exists.
    choices: ['present', 'absent']
    default: 'present'
    type: str
  name:
    description:
      - Name of the realm key to create.
    type: str
    required: true
  force:
    description:
      - Enforce the state of the private key and certificate. This is not automatically the case as this module is unable
        to determine the current state of the private key and thus cannot trigger an update based on an actual divergence.
        That said, a private key update may happen even if force is false as a side-effect of other changes.
    default: false
    type: bool
  parent_id:
    description:
      - The parent_id of the realm key. In practice the name of the realm.
    type: str
    required: true
  provider_id:
    description:
      - The name of the "provider ID" for the key.
      - The value V(rsa-enc) has been added in community.general 8.2.0.
      - The value V(java-keystore) has been added in community.general 12.4.0. This provider imports keys from
        a Java Keystore (JKS or PKCS12) file located on the Keycloak server filesystem.
      - The values V(rsa-generated), V(hmac-generated), V(aes-generated), and V(ecdsa-generated) have been added in
        community.general 12.4.0. These are auto-generated key providers where Keycloak manages the key material.
      - The values V(rsa-enc-generated), V(ecdh-generated), and V(eddsa-generated) have been added in
        community.general 12.4.0. These correspond to the auto-generated key providers available in Keycloak 26.
    choices:
      - rsa
      - rsa-enc
      - java-keystore
      - rsa-generated
      - rsa-enc-generated
      - hmac-generated
      - aes-generated
      - ecdsa-generated
      - ecdh-generated
      - eddsa-generated
    default: 'rsa'
    type: str
  config:
    description:
      - Dict specifying the key and its properties.
    type: dict
    suboptions:
      active:
        description:
          - Whether they key is active or inactive. Not to be confused with the state of the Ansible resource managed by the
            O(state) parameter.
        default: true
        type: bool
      enabled:
        description:
          - Whether the key is enabled or disabled. Not to be confused with the state of the Ansible resource managed by the
            O(state) parameter.
        default: true
        type: bool
      priority:
        description:
          - The priority of the key.
        type: int
        required: true
      algorithm:
        description:
          - Key algorithm.
          - The values V(RS384), V(RS512), V(PS256), V(PS384), V(PS512), V(RSA1_5), V(RSA-OAEP), V(RSA-OAEP-256) have been
            added in community.general 8.2.0.
          - The values V(HS256), V(HS384), V(HS512) (for HMAC), V(ES256), V(ES384), V(ES512) (for ECDSA), and V(AES)
            have been added in community.general 12.4.0.
          - The values V(ECDH_ES), V(ECDH_ES_A128KW), V(ECDH_ES_A192KW), V(ECDH_ES_A256KW) (for ECDH key exchange),
            and V(Ed25519), V(Ed448) (for EdDSA signing) have been added in community.general 12.4.0.
          - For O(provider_id=rsa), O(provider_id=rsa-generated), and O(provider_id=java-keystore), defaults to V(RS256).
          - For O(provider_id=rsa-enc) and O(provider_id=rsa-enc-generated), must be one of V(RSA1_5), V(RSA-OAEP), V(RSA-OAEP-256) (required, no default).
          - For O(provider_id=hmac-generated), must be one of V(HS256), V(HS384), V(HS512) (required, no default).
          - For O(provider_id=ecdsa-generated), must be one of V(ES256), V(ES384), V(ES512) (required, no default).
          - For O(provider_id=ecdh-generated), must be one of V(ECDH_ES), V(ECDH_ES_A128KW), V(ECDH_ES_A192KW), V(ECDH_ES_A256KW) (required, no default).
          - For O(provider_id=eddsa-generated), this option is not used (the algorithm is determined by O(config.elliptic_curve)).
          - For O(provider_id=aes-generated), this option is not used (AES is always used).
        choices:
          - RS256
          - RS384
          - RS512
          - PS256
          - PS384
          - PS512
          - RSA1_5
          - RSA-OAEP
          - RSA-OAEP-256
          - HS256
          - HS384
          - HS512
          - ES256
          - ES384
          - ES512
          - AES
          - ECDH_ES
          - ECDH_ES_A128KW
          - ECDH_ES_A192KW
          - ECDH_ES_A256KW
          - Ed25519
          - Ed448
        default: RS256
        type: str
      private_key:
        description:
          - The private key as an ASCII string. Contents of the key must match O(config.algorithm) and O(provider_id).
          - Please note that the module cannot detect whether the private key specified differs from the current state's private
            key. Use O(force=true) to force the module to update the private key if you expect it to be updated.
          - Required when O(provider_id) is V(rsa) or V(rsa-enc). Not used for auto-generated providers.
        type: str
      certificate:
        description:
          - A certificate signed with the private key as an ASCII string. Contents of the key must match O(config.algorithm)
            and O(provider_id).
          - If you want Keycloak to automatically generate a certificate using your private key then set this to an empty
            string.
          - Required when O(provider_id) is V(rsa) or V(rsa-enc). Not used for auto-generated providers.
        type: str
      secret_size:
        description:
          - The size of the generated secret key in bytes.
          - Only applicable to O(provider_id=hmac-generated) and O(provider_id=aes-generated).
          - Valid values are V(16), V(24), V(32), V(64), V(128), V(256), V(512).
          - Default is V(64) for HMAC, V(16) for AES.
        type: int
        version_added: 12.4.0
      key_size:
        description:
          - The size of the generated key in bits.
          - Only applicable to O(provider_id=rsa-generated) and O(provider_id=rsa-enc-generated).
          - Valid values are V(1024), V(2048), V(4096). Default is V(2048).
        type: int
        version_added: 12.4.0
      elliptic_curve:
        description:
          - The elliptic curve to use for ECDSA, ECDH, or EdDSA keys.
          - For O(provider_id=ecdsa-generated) and O(provider_id=ecdh-generated), valid values are V(P-256), V(P-384), V(P-521). Default is V(P-256).
          - For O(provider_id=eddsa-generated), valid values are V(Ed25519), V(Ed448). Default is V(Ed25519).
        type: str
        choices: ['P-256', 'P-384', 'P-521', 'Ed25519', 'Ed448']
        version_added: 12.4.0
      keystore:
        description:
          - Path to the Java Keystore file on the Keycloak server filesystem.
          - Required when O(provider_id=java-keystore).
        type: str
        version_added: 12.4.0
      keystore_password:
        description:
          - Password for the Java Keystore.
          - Required when O(provider_id=java-keystore).
        type: str
        version_added: 12.4.0
      key_alias:
        description:
          - Alias of the key within the keystore.
          - Required when O(provider_id=java-keystore).
        type: str
        version_added: 12.4.0
      key_password:
        description:
          - Password for the key within the keystore.
          - If not specified, the O(config.keystore_password) is used.
          - Only applicable to O(provider_id=java-keystore).
        type: str
        version_added: 12.4.0
  update_password:
    description:
      - Controls when passwords are sent to Keycloak for V(java-keystore) provider.
      - V(always) - Always send passwords. Keycloak will update the component even if passwords
        have not changed. Use when you need to ensure passwords are updated.
      - V(on_create) - Only send passwords when creating a new component. When updating an
        existing component, send the masked value to preserve existing passwords. This makes
        the module idempotent for password fields.
      - This is necessary because Keycloak masks passwords in API responses (returns C(**********)),
        making comparison impossible.
      - Has no effect for providers other than V(java-keystore).
    type: str
    choices: ['always', 'on_create']
    default: always
    version_added: 12.4.0
notes:
  - Current value of the private key cannot be fetched from Keycloak. Therefore comparing its desired state to the current
    state is not possible.
  - If O(config.certificate) is not explicitly provided it is dynamically created by Keycloak. Therefore comparing the current
    state of the certificate to the desired state (which may be empty) is not possible.
  - Due to the private key and certificate options the module is B(not fully idempotent). You can use O(force=true) to force
    the module to ensure updating if you know that the private key might have changed.
  - For auto-generated providers (V(rsa-generated), V(rsa-enc-generated), V(hmac-generated), V(aes-generated), V(ecdsa-generated),
    V(ecdh-generated), V(eddsa-generated)), Keycloak manages the key material automatically. The O(config.private_key) and
    O(config.certificate) options are not used.
  - For V(java-keystore) provider, the O(config.keystore_password) and O(config.key_password) values are returned masked by
    Keycloak. Therefore comparing their current state to the desired state is not possible. Use O(update_password=on_create)
    for idempotent playbooks, or use O(update_password=always) (default) if you need to ensure passwords are updated.
extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Samuli Seppänen (@mattock)
  - Ivan Kokalović (@koke1997)
"""

EXAMPLES = r"""
- name: Manage Keycloak realm key (certificate autogenerated by Keycloak)
  community.general.keycloak_realm_key:
    name: custom
    state: present
    parent_id: master
    provider_id: rsa
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      private_key: "{{ private_key }}"
      certificate: ""
      enabled: true
      active: true
      priority: 120
      algorithm: RS256

- name: Manage Keycloak realm key and certificate
  community.general.keycloak_realm_key:
    name: custom
    state: present
    parent_id: master
    provider_id: rsa
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      private_key: "{{ private_key }}"
      certificate: "{{ certificate }}"
      enabled: true
      active: true
      priority: 120
      algorithm: RS256

- name: Create HMAC signing key (auto-generated)
  community.general.keycloak_realm_key:
    name: hmac-custom
    state: present
    parent_id: master
    provider_id: hmac-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: HS256
      secret_size: 64

- name: Create AES encryption key (auto-generated)
  community.general.keycloak_realm_key:
    name: aes-custom
    state: present
    parent_id: master
    provider_id: aes-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      secret_size: 16

- name: Create ECDSA signing key (auto-generated)
  community.general.keycloak_realm_key:
    name: ecdsa-custom
    state: present
    parent_id: master
    provider_id: ecdsa-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: ES256
      elliptic_curve: P-256

- name: Create RSA signing key (auto-generated)
  community.general.keycloak_realm_key:
    name: rsa-auto
    state: present
    parent_id: master
    provider_id: rsa-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: RS256
      key_size: 2048

- name: Remove default HMAC key
  community.general.keycloak_realm_key:
    name: hmac-generated
    state: absent
    parent_id: myrealm
    provider_id: hmac-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      priority: 100

- name: Create RSA encryption key (auto-generated)
  community.general.keycloak_realm_key:
    name: rsa-enc-auto
    state: present
    parent_id: master
    provider_id: rsa-enc-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: RSA-OAEP
      key_size: 2048

- name: Create ECDH key exchange key (auto-generated)
  community.general.keycloak_realm_key:
    name: ecdh-custom
    state: present
    parent_id: master
    provider_id: ecdh-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: ECDH_ES
      elliptic_curve: P-256

- name: Create EdDSA signing key (auto-generated)
  community.general.keycloak_realm_key:
    name: eddsa-custom
    state: present
    parent_id: master
    provider_id: eddsa-generated
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    config:
      enabled: true
      active: true
      priority: 100
      elliptic_curve: Ed25519

- name: Import key from Java Keystore (always update passwords)
  community.general.keycloak_realm_key:
    name: jks-imported
    state: present
    parent_id: master
    provider_id: java-keystore
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    # update_password: always is the default - passwords are always sent to Keycloak
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: RS256
      keystore: /opt/keycloak/conf/keystore.jks
      keystore_password: "{{ keystore_password }}"
      key_alias: mykey
      key_password: "{{ key_password }}"

- name: Import key from Java Keystore (idempotent - only set password on create)
  community.general.keycloak_realm_key:
    name: jks-idempotent
    state: present
    parent_id: master
    provider_id: java-keystore
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
    update_password: on_create  # Only send passwords when creating, preserve existing on update
    config:
      enabled: true
      active: true
      priority: 100
      algorithm: RS256
      keystore: /opt/keycloak/conf/keystore.jks
      keystore_password: "{{ keystore_password }}"
      key_alias: mykey
      key_password: "{{ key_password }}"
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str

end_state:
  description: Representation of the keycloak_realm_key after module execution.
  returned: on success
  type: dict
  contains:
    id:
      description: ID of the realm key.
      type: str
      returned: when O(state=present)
      sample: 5b7ec13f-99da-46ad-8326-ab4c73cf4ce4
    name:
      description: Name of the realm key.
      type: str
      returned: when O(state=present)
      sample: mykey
    parentId:
      description: ID of the realm this key belongs to.
      type: str
      returned: when O(state=present)
      sample: myrealm
    providerId:
      description: The ID of the key provider.
      type: str
      returned: when O(state=present)
      sample: rsa
    providerType:
      description: The type of provider.
      type: str
      returned: when O(state=present)
    config:
      description: Realm key configuration.
      type: dict
      returned: when O(state=present)
      sample:
        {
          "active": [
            "true"
          ],
          "algorithm": [
            "RS256"
          ],
          "enabled": [
            "true"
          ],
          "priority": [
            "140"
          ]
        }
    key_info:
      description:
        - Cryptographic key metadata fetched from the realm keys endpoint.
        - Only returned for V(java-keystore) provider when O(state=present) and not in check mode.
        - This includes the key ID (kid) and certificate fingerprint, which can be used to detect
          if the actual cryptographic key changed.
      type: dict
      returned: when O(provider_id=java-keystore) and O(state=present)
      version_added: 12.4.0
      contains:
        kid:
          description: The key ID (kid) - unique identifier for the cryptographic key.
          type: str
          sample: bN7p5Nc_V2M7N_-mb5vVSRVPKq5qD_OuARInB9ofsJ0
        certificate_fingerprint:
          description: SHA256 fingerprint of the certificate in colon-separated hex format.
          type: str
          sample: "A1:B2:C3:D4:E5:F6:..."
        status:
          description: The key status (ACTIVE, PASSIVE, DISABLED).
          type: str
          sample: ACTIVE
        valid_to:
          description: Certificate expiration timestamp in milliseconds since epoch.
          type: int
          sample: 1801789047000
"""

import base64
import binascii
import hashlib
from copy import deepcopy
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    camel,
    get_token,
    keycloak_argument_spec,
)

# Provider IDs that require private_key and certificate
IMPORTED_KEY_PROVIDERS = ["rsa", "rsa-enc"]
# Provider IDs that import keys from Java Keystore
KEYSTORE_PROVIDERS = ["java-keystore"]
# Provider IDs that auto-generate keys
GENERATED_KEY_PROVIDERS = [
    "rsa-generated",
    "rsa-enc-generated",
    "hmac-generated",
    "aes-generated",
    "ecdsa-generated",
    "ecdh-generated",
    "eddsa-generated",
]

# Mapping of Ansible parameter names to Keycloak config property names
# for cases where camel() conversion doesn't produce the correct result.
# Each provider type may use a different config key for elliptic curve.
CONFIG_PARAM_MAPPING = {
    "elliptic_curve": "ecdsaEllipticCurveKey",
}

# Provider-specific config key names for elliptic_curve parameter
# ECDSA and ECDH both use the same curves (P-256, P-384, P-521) but different config keys
# EdDSA uses different curves (Ed25519, Ed448) with its own config key
ELLIPTIC_CURVE_CONFIG_KEYS = {
    "ecdsa-generated": "ecdsaEllipticCurveKey",
    "ecdh-generated": "ecdhEllipticCurveKey",
    "eddsa-generated": "eddsaEllipticCurveKey",
}

# Valid algorithm choices per provider type
# Note: aes-generated and eddsa-generated don't use algorithm config
PROVIDER_ALGORITHMS = {
    "rsa": ["RS256", "RS384", "RS512", "PS256", "PS384", "PS512"],
    "rsa-enc": ["RSA1_5", "RSA-OAEP", "RSA-OAEP-256"],
    "java-keystore": ["RS256", "RS384", "RS512", "PS256", "PS384", "PS512"],
    "rsa-generated": ["RS256", "RS384", "RS512", "PS256", "PS384", "PS512"],
    "rsa-enc-generated": ["RSA1_5", "RSA-OAEP", "RSA-OAEP-256"],
    "hmac-generated": ["HS256", "HS384", "HS512"],
    "ecdsa-generated": ["ES256", "ES384", "ES512"],
    "ecdh-generated": ["ECDH_ES", "ECDH_ES_A128KW", "ECDH_ES_A192KW", "ECDH_ES_A256KW"],
}

# Providers that don't use the algorithm config parameter
# eddsa-generated: algorithm is determined by the elliptic curve (Ed25519 or Ed448)
# aes-generated: always uses AES algorithm
PROVIDERS_WITHOUT_ALGORITHM = ["aes-generated", "eddsa-generated"]

# Providers where the RS256 default is valid (for backward compatibility)
PROVIDERS_WITH_RS256_DEFAULT = ["rsa", "rsa-generated", "java-keystore"]

# Config keys that cannot be compared and must be removed from changesets/diffs.
# privateKey/certificate: Keycloak doesn't return private keys, certificates are generated dynamically.
# keystorePassword/keyPassword: Keycloak masks these with "**********" in API responses.
SENSITIVE_CONFIG_KEYS = ["privateKey", "certificate", "keystorePassword", "keyPassword"]


def remove_sensitive_config_keys(config):
    for key in SENSITIVE_CONFIG_KEYS:
        config.pop(key, None)


def get_keycloak_config_key(param_name, provider_id=None):
    """Convert Ansible parameter name to Keycloak config key.

    Uses explicit mapping if available, otherwise applies camelCase conversion.
    For elliptic_curve, the config key depends on the provider type.
    """
    # Handle elliptic_curve specially - each provider uses a different config key
    if param_name == "elliptic_curve" and provider_id in ELLIPTIC_CURVE_CONFIG_KEYS:
        return ELLIPTIC_CURVE_CONFIG_KEYS[param_name]
    if param_name in CONFIG_PARAM_MAPPING:
        return CONFIG_PARAM_MAPPING[param_name]
    return camel(param_name)


def compute_certificate_fingerprint(certificate_pem):
    try:
        cert_der = base64.b64decode(certificate_pem)
        fingerprint = hashlib.sha256(cert_der).hexdigest().upper()
        return ":".join(fingerprint[i : i + 2] for i in range(0, len(fingerprint), 2))
    except (ValueError, binascii.Error, TypeError):
        return None


def get_key_info_for_component(kc, realm, component_id):
    try:
        keys_response = kc.get_realm_keys_metadata_by_id(realm)
        if not keys_response or "keys" not in keys_response:
            return None

        for key in keys_response.get("keys", []):
            if key.get("providerId") == component_id:
                return {
                    "kid": key.get("kid"),
                    "certificate_fingerprint": compute_certificate_fingerprint(key.get("certificate")),
                    "public_key": key.get("publicKey"),
                    "valid_to": key.get("validTo"),
                    "status": key.get("status"),
                    "algorithm": key.get("algorithm"),
                    "type": key.get("type"),
                }
        return None
    except (KeyError, TypeError):
        return None


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        name=dict(type="str", required=True),
        force=dict(type="bool", default=False),
        parent_id=dict(type="str", required=True),
        provider_id=dict(
            type="str",
            default="rsa",
            choices=[
                "rsa",
                "rsa-enc",
                "java-keystore",
                "rsa-generated",
                "rsa-enc-generated",
                "hmac-generated",
                "aes-generated",
                "ecdsa-generated",
                "ecdh-generated",
                "eddsa-generated",
            ],
        ),
        config=dict(
            type="dict",
            options=dict(
                active=dict(type="bool", default=True),
                enabled=dict(type="bool", default=True),
                priority=dict(type="int", required=True),
                algorithm=dict(
                    type="str",
                    default="RS256",
                    choices=[
                        "RS256",
                        "RS384",
                        "RS512",
                        "PS256",
                        "PS384",
                        "PS512",
                        "RSA1_5",
                        "RSA-OAEP",
                        "RSA-OAEP-256",
                        "HS256",
                        "HS384",
                        "HS512",
                        "ES256",
                        "ES384",
                        "ES512",
                        "AES",
                        "ECDH_ES",
                        "ECDH_ES_A128KW",
                        "ECDH_ES_A192KW",
                        "ECDH_ES_A256KW",
                        "Ed25519",
                        "Ed448",
                    ],
                ),
                private_key=dict(type="str", no_log=True),
                certificate=dict(type="str"),
                secret_size=dict(type="int", no_log=False),
                key_size=dict(type="int"),
                elliptic_curve=dict(type="str", choices=["P-256", "P-384", "P-521", "Ed25519", "Ed448"]),
                keystore=dict(type="str", no_log=False),
                keystore_password=dict(type="str", no_log=True),
                key_alias=dict(type="str", no_log=False),
                key_password=dict(type="str", no_log=True),
            ),
        ),
        update_password=dict(
            type="str",
            default="always",
            choices=["always", "on_create"],
            no_log=False,
        ),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    provider_id = module.params["provider_id"]
    config = module.params["config"] or {}
    state = module.params["state"]

    # Validate that imported key providers have the required parameters
    if state == "present" and provider_id in IMPORTED_KEY_PROVIDERS:
        if not config.get("private_key"):
            module.fail_json(msg=f"config.private_key is required for provider_id '{provider_id}'")
        if config.get("certificate") is None:
            module.fail_json(
                msg=f"config.certificate is required for provider_id '{provider_id}' (use empty string for auto-generation)"
            )

    # Validate that java-keystore providers have the required parameters
    if state == "present" and provider_id in KEYSTORE_PROVIDERS:
        required_params = ["keystore", "keystore_password", "key_alias"]
        missing = [p for p in required_params if not config.get(p)]
        if missing:
            module.fail_json(
                msg=f"For provider_id=java-keystore, the following config options are required: {', '.join(missing)}"
            )

    # Validate algorithm for providers that use it
    if state == "present":
        algorithm = config.get("algorithm")
        if provider_id in PROVIDER_ALGORITHMS:
            valid_algorithms = PROVIDER_ALGORITHMS[provider_id]
            if algorithm not in valid_algorithms:
                msg = f"algorithm '{algorithm}' is not valid for provider_id '{provider_id}'."
                if algorithm == "RS256" and provider_id not in PROVIDERS_WITH_RS256_DEFAULT:
                    msg += " The default 'RS256' is not valid for this provider."
                msg += f" Valid choices are: {', '.join(valid_algorithms)}"
                module.fail_json(msg=msg)
        elif provider_id in PROVIDERS_WITHOUT_ALGORITHM and algorithm is not None and algorithm != "RS256":
            # aes-generated and eddsa-generated don't use algorithm - only warn if user explicitly set a non-default value
            module.warn(f"algorithm is ignored for provider_id '{provider_id}'")

    # Validate elliptic curve for providers that use it
    if state == "present":
        elliptic_curve = config.get("elliptic_curve")
        if provider_id in ["ecdsa-generated", "ecdh-generated"] and elliptic_curve is not None:
            valid_curves = ["P-256", "P-384", "P-521"]
            if elliptic_curve not in valid_curves:
                module.fail_json(
                    msg=f"elliptic_curve '{elliptic_curve}' is not valid for provider_id '{provider_id}'. "
                    f"Valid choices are: {', '.join(valid_curves)}"
                )
        elif provider_id == "eddsa-generated" and elliptic_curve is not None:
            valid_curves = ["Ed25519", "Ed448"]
            if elliptic_curve not in valid_curves:
                module.fail_json(
                    msg=f"elliptic_curve '{elliptic_curve}' is not valid for provider_id '{provider_id}'. "
                    f"Valid choices are: {', '.join(valid_curves)}"
                )

    result = dict(changed=False, msg="", end_state={}, diff=dict(before={}, after={}))

    # This will include the current state of the realm key if it is already
    # present. This is only used for diff-mode.
    before_realm_key = {}
    before_realm_key["config"] = {}

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    params_to_ignore = list(keycloak_argument_spec().keys()) + ["state", "force", "parent_id", "update_password"]

    # Filter and map the parameters names that apply to the role
    component_params = [x for x in module.params if x not in params_to_ignore and module.params.get(x) is not None]

    # We only support one component provider type in this module
    provider_type = "org.keycloak.keys.KeyProvider"

    # Build a proposed changeset from parameters given to this module
    changeset = {}
    changeset["config"] = {}

    # Generate a JSON payload for Keycloak Admin API from the module
    # parameters.  Parameters that do not belong to the JSON payload (e.g.
    # "state" or "auth_keycloal_url") have been filtered away earlier (see
    # above).
    #
    # This loop converts Ansible module parameters (snake-case) into
    # Keycloak-compatible format (camel-case). For example private_key
    # becomes privateKey.
    #
    # It also converts bool, str and int parameters into lists with a single
    # entry of 'str' type. Bool values are also lowercased. This is required
    # by Keycloak.
    #
    for component_param in component_params:
        if component_param == "config":
            for config_param in module.params["config"]:
                raw_value = module.params["config"][config_param]
                # Optional params (secret_size, key_size, elliptic_curve) default to None.
                # Skip them to avoid sending str(None) = "None" as a config value to Keycloak.
                if raw_value is None:
                    continue
                # Use custom mapping if available, otherwise camelCase
                # Pass provider_id for elliptic_curve which uses different config keys per provider
                keycloak_key = get_keycloak_config_key(config_param, provider_id)
                changeset["config"][keycloak_key] = []
                if isinstance(raw_value, bool):
                    value = str(raw_value).lower()
                else:
                    value = str(raw_value)

                changeset["config"][keycloak_key].append(value)
        else:
            # No need for camelcase in here as these are one word parameters
            new_param_value = module.params[component_param]
            changeset[camel(component_param)] = new_param_value

    # As provider_type is not a module parameter we have to add it to the
    # changeset explicitly.
    changeset["providerType"] = provider_type

    # Make a deep copy of the changeset. This is use when determining
    # changes to the current state.
    changeset_copy = deepcopy(changeset)

    # Remove keys that cannot be compared: privateKey/certificate (not returned
    # by Keycloak API) and keystore passwords (masked with "**********").
    # The actual values remain in 'changeset' for the API payload.
    remove_sensitive_config_keys(changeset_copy["config"])

    name = module.params["name"]
    force = module.params["force"]
    parent_id = module.params["parent_id"]

    # Get a list of all Keycloak components that are of keyprovider type.
    realm_keys = kc.get_components(urlencode(dict(type=provider_type)), parent_id)

    # If this component is present get its key ID. Confusingly the key ID is
    # also known as the Provider ID.
    key_id = None

    # Track individual parameter changes
    changes = ""

    # This tells Ansible whether the key was changed (added, removed, modified)
    result["changed"] = False

    # Loop through the list of components. If we encounter a component whose
    # name matches the value of the name parameter then assume the key is
    # already present.
    for key in realm_keys:
        if key["name"] == name:
            key_id = key["id"]
            changeset["id"] = key_id
            changeset_copy["id"] = key_id

            # Compare top-level parameters
            for param in changeset:
                before_realm_key[param] = key[param]

                if changeset_copy[param] != key[param] and param != "config":
                    changes += f"{param}: {key[param]} -> {changeset_copy[param]}, "
                    result["changed"] = True

            # Compare parameters under the "config" key
            # Note: Keycloak API may not return all config fields for default keys
            # (e.g., 'active', 'enabled', 'algorithm' may be missing). Handle this
            # gracefully by using .get() with defaults.
            for p, v in changeset_copy["config"].items():
                # Get the current value, defaulting to our expected value if not present
                # This handles the case where Keycloak doesn't return certain fields
                # for default/generated keys
                current_value = key["config"].get(p, v)
                before_realm_key["config"][p] = current_value
                if v != current_value:
                    changes += f"config.{p}: {current_value} -> {v}, "
                    result["changed"] = True

            # For java-keystore provider, also fetch and compare key info (kid)
            # This detects if the actual cryptographic key changed even when
            # other config parameters remain the same
            if provider_id in KEYSTORE_PROVIDERS:
                current_key_info = get_key_info_for_component(kc, parent_id, key_id)
                if current_key_info:
                    before_realm_key["key_info"] = {
                        "kid": current_key_info.get("kid"),
                        "certificate_fingerprint": current_key_info.get("certificate_fingerprint"),
                    }

    # Sanitize linefeeds for the privateKey and certificate (only for imported providers).
    # Without this the JSON payload will be invalid.
    if "privateKey" in changeset["config"]:
        changeset["config"]["privateKey"][0] = changeset["config"]["privateKey"][0].replace("\\n", "\n")
    if "certificate" in changeset["config"]:
        changeset["config"]["certificate"][0] = changeset["config"]["certificate"][0].replace("\\n", "\n")

    # For java-keystore provider: handle update_password parameter
    # When update_password=on_create and we're updating an existing component,
    # replace actual passwords with the masked value ("**********") that Keycloak
    # returns in API responses. When Keycloak receives this masked value, it
    # preserves the existing password instead of updating it.
    # This makes the module idempotent for password fields.
    update_password = module.params["update_password"]
    if provider_id in KEYSTORE_PROVIDERS and key_id and update_password == "on_create":
        SECRET_VALUE = "**********"
        if "keystorePassword" in changeset["config"]:
            changeset["config"]["keystorePassword"] = [SECRET_VALUE]
        if "keyPassword" in changeset["config"]:
            changeset["config"]["keyPassword"] = [SECRET_VALUE]

    # Check all the possible states of the resource and do what is needed to
    # converge current state with desired state (create, update or delete
    # the key).
    if key_id and state == "present":
        if result["changed"]:
            if module._diff:
                remove_sensitive_config_keys(before_realm_key["config"])
                result["diff"] = dict(before=before_realm_key, after=changeset_copy)

            if module.check_mode:
                result["msg"] = f"Realm key {name} would be changed: {changes.strip(', ')}"
            else:
                kc.update_component(changeset, parent_id)
                result["msg"] = f"Realm key {name} changed: {changes.strip(', ')}"
        elif not result["changed"] and force:
            kc.update_component(changeset, parent_id)
            result["changed"] = True
            result["msg"] = f"Realm key {name} was forcibly updated"
        else:
            result["msg"] = f"Realm key {name} was in sync"

        result["end_state"] = changeset_copy

        # For java-keystore provider, include key info in end_state
        if provider_id in KEYSTORE_PROVIDERS:
            if not module.check_mode:
                key_info = get_key_info_for_component(kc, parent_id, key_id)
                if key_info:
                    result["end_state"]["key_info"] = {
                        "kid": key_info.get("kid"),
                        "certificate_fingerprint": key_info.get("certificate_fingerprint"),
                        "status": key_info.get("status"),
                        "valid_to": key_info.get("valid_to"),
                    }
                else:
                    module.warn(
                        f"Key component '{name}' exists but no active key was found. "
                        "This may indicate an incorrect keystore password, path, or alias."
                    )
    elif key_id and state == "absent":
        if module._diff:
            remove_sensitive_config_keys(before_realm_key["config"])
            result["diff"] = dict(before=before_realm_key, after={})

        if module.check_mode:
            result["changed"] = True
            result["msg"] = f"Realm key {name} would be deleted"
        else:
            kc.delete_component(key_id, parent_id)
            result["changed"] = True
            result["msg"] = f"Realm key {name} deleted"

        result["end_state"] = {}
    elif not key_id and state == "present":
        if module._diff:
            result["diff"] = dict(before={}, after=changeset_copy)

        if module.check_mode:
            result["changed"] = True
            result["msg"] = f"Realm key {name} would be created"
        else:
            kc.create_component(changeset, parent_id)
            result["changed"] = True
            result["msg"] = f"Realm key {name} created"

            # For java-keystore provider, fetch and include key info after creation
            if provider_id in KEYSTORE_PROVIDERS:
                # We need to get the component ID first (it was just created)
                realm_keys_after = kc.get_components(urlencode(dict(type=provider_type)), parent_id)
                for k in realm_keys_after:
                    if k["name"] == name:
                        new_key_id = k["id"]
                        key_info = get_key_info_for_component(kc, parent_id, new_key_id)
                        if key_info:
                            changeset_copy["key_info"] = {
                                "kid": key_info.get("kid"),
                                "certificate_fingerprint": key_info.get("certificate_fingerprint"),
                                "status": key_info.get("status"),
                                "valid_to": key_info.get("valid_to"),
                            }
                        else:
                            module.warn(
                                f"Key component '{name}' was created but no active key was found. "
                                "This may indicate an incorrect keystore password, path, or alias."
                            )
                        break

        result["end_state"] = changeset_copy
    elif not key_id and state == "absent":
        result["changed"] = False
        result["msg"] = f"Realm key {name} not present"
        result["end_state"] = {}

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_http_proxy
short_description: Manages proxy settings for WinHTTP
description:
- Used to set, remove, or import proxy settings for Windows HTTP Services
  C(WinHTTP).
- WinHTTP is a framework used by applications or services, typically .NET
  applications or non-interactive services, to make web requests.
version_added: 2.7.0
options:
  bypass:
    description:
    - A list of hosts that will bypass the set proxy when being accessed.
    - Use C(<local>) to match hostnames that are not fully qualified domain
      names. This is useful when needing to connect to intranet sites using
      just the hostname.
    - Omit, set to null or an empty string/list to remove the bypass list.
    - If this is set then I(proxy) must also be set.
    type: list
    elements: str
  proxy:
    description:
    - A string or dict that specifies the proxy to be set.
    - If setting a string, should be in the form C(hostname), C(hostname:port),
      or C(protocol=hostname:port).
    - If the port is undefined, the default port for the protocol in use is
      used.
    - If setting a dict, the keys should be the protocol and the values should
      be the hostname and/or port for that protocol.
    - Valid protocols are C(http), C(https), C(ftp), and C(socks).
    - Omit, set to null or an empty string to remove the proxy settings.
    type: raw
  source:
    description:
    - Instead of manually specifying the I(proxy) and/or I(bypass), set this to
      import the proxy from a set source like Internet Explorer.
    - Using C(ie) will import the Internet Explorer proxy settings for the
      current active network connection of the current user.
    - Only IE's proxy URL and bypass list will be imported into WinHTTP.
    - This is like running C(netsh winhttp import proxy source=ie).
    - The value is imported when the module runs and will not automatically
      be updated if the IE configuration changes in the future. The module will
      have to be run again to sync the latest changes.
    choices:
    - ie
    type: str
notes:
- This is not the same as the proxy settings set in Internet Explorer, also
  known as C(WinINet); use the M(ansible.windows.win_inet_proxy) module to manage that instead.
- These settings are set system wide and not per user, it will require
  Administrative privileges to run.
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: Set a proxy to use for all protocols
  ansible.windows.win_http_proxy:
    proxy: hostname

- name: Set a proxy with a specific port with a bypass list
  ansible.windows.win_http_proxy:
    proxy: hostname:8080
    bypass:
      - server1
      - server2
      - <local>

- name: Set the proxy based on the IE proxy settings
  ansible.windows.win_http_proxy:
    source: ie

- name: Set a proxy for specific protocols
  ansible.windows.win_http_proxy:
    proxy:
      http: hostname:8080
      https: hostname:8443

- name: Set a proxy for specific protocols using a string
  ansible.windows.win_http_proxy:
    proxy: http=hostname:8080;https=hostname:8443
    bypass: server1,server2,<local>

- name: Remove any proxy settings
  ansible.windows.win_http_proxy:
    proxy: ''
    bypass: ''
'''

RETURN = r'''
#
'''



# Copyright (c) 2014, Kevin Carter <kevin.carter@rackspace.com>
# Copyright (c) 2025, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import subprocess
import tempfile

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_bytes

# This is used to attach to a running container and execute commands from
# within the container on the host.  This will provide local access to a
# container without using SSH.  The template will attempt to work within the
# home directory of the user that was attached to the container and source
# that users environment variables by default.
ATTACH_TEMPLATE = """#!/usr/bin/env bash
pushd "$(getent passwd $(whoami)|cut -f6 -d':')"
    if [[ -f ".bashrc" ]];then
        source .bashrc
        unset HOSTNAME
    fi
popd

# User defined command
{}
"""


def create_script(arg_tuple: tuple[str, AnsibleModule]) -> None:
    """Write out a script onto a target.

    This method should be backward compatible with Python when executing
    from within the container.

    :param arg_tuple: a tuple of (command, module) where command is the command
                      to run (this can be a script and can use spacing with
                      newlines as separation) and module is the AnsibleModule
                      to run commands with.
    """

    command, module = arg_tuple

    script_file = ""
    try:
        f = tempfile.NamedTemporaryFile(prefix="lxc-attach-script", delete=False, mode="wb")
        f.write(to_bytes(ATTACH_TEMPLATE.format(command), errors="surrogate_or_strict"))
        script_file = f.name
        f.flush()
        f.close()

        os.chmod(script_file, 0o0700)

        with tempfile.NamedTemporaryFile(prefix="lxc-attach-script-log", delete=False, mode="ab") as stdout_file:
            with tempfile.NamedTemporaryFile(prefix="lxc-attach-script-err", delete=False, mode="ab") as stderr_file:
                subprocess.Popen([script_file], stdout=stdout_file, stderr=stderr_file).communicate()

    finally:
        if script_file:
            os.remove(script_file)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: network_in_usable
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import (
    _address_normalizer,
    _need_netaddr,
    _range_checker,
    ipaddr,
)


__metaclass__ = type


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

DOCUMENTATION = """
    name: network_in_usable
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: The network_in_usable filter returns whether an address passed as an argument is usable in a network.
    description:
    - The network_in_usable filter returns whether an address passed as an argument is usable in a network
      Usable addresses are addresses that can be assigned to a host.
    - The network ID and the broadcast address are not usable addresses.
    options:
        value:
            description: The network address or range to test against.
            type: str
            required: True
        test:
            description: The address or network is usable or not.
            type: str
    notes:
"""

EXAMPLES = r"""
#### examples
- name: Check ip address is usable in a network
  debug:
    msg: "{{ '192.168.0.0/24' | ansible.utils.network_in_usable( '192.168.0.1' ) }}"

- name: Check broadcast address is usable in a network
  debug:
    msg: "{{ '192.168.0.0/24' | ansible.utils.network_in_usable( '192.168.0.255' ) }}"

- name: Check in a network is part of another network.
  debug:
    msg: "{{ '192.168.0.0/16' | ansible.utils.network_in_usable( '192.168.0.255' ) }}"

# TASK [Check ip address is usable in a network] **************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_network_in_usable.yaml:7
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": true
# }
#
# TASK [Check broadcast address is usable in a network] *******************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_network_in_usable.yaml:11
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": false
# }
#
# TASK [Check in a network is part of another network.] *******************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_network_in_usable.yaml:15
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": true
# }
"""

RETURN = """
  data:
    type: bool
    description:
      - Returns whether an address or a network passed as argument is in a network.

"""


@pass_environment
def _network_in_usable(*args, **kwargs):
    """This filter returns whether an address or a network passed as argument is in a network."""
    keys = ["value", "test"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="network_in_usable")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return network_in_usable(**updated_data)


def network_in_usable(value, test):
    """
    Checks whether 'test' is a usable address or addresses in 'value'
    :param: value: The string representation of an address or network to test against.
    :param test: The string representation of an address or network to validate if it is within the range of 'value'.
    :return: bool
    """
    # normalize value and test variables into an ipaddr
    v = _address_normalizer(value)
    w = _address_normalizer(test)

    # get first and last addresses as integers to compare value and test; or cathes value when case is /32
    v_first = ipaddr(ipaddr(v, "first_usable") or ipaddr(v, "address"), "int")
    v_last = ipaddr(ipaddr(v, "last_usable") or ipaddr(v, "address"), "int")
    w_first = ipaddr(ipaddr(w, "network") or ipaddr(w, "address"), "int")
    w_last = ipaddr(ipaddr(w, "broadcast") or ipaddr(w, "address"), "int")

    if _range_checker(w_first, v_first, v_last) and _range_checker(w_last, v_first, v_last):
        return True
    else:
        return False


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "network_in_usable": _network_in_usable,
    }

    def filters(self):
        """ipaddr filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_contract_service_graph
short_description: Manage the service graph association with a contract in schema template
description:
- Manage the service graph association with a contract in schema template on Cisco ACI Multi-Site.
- The Contract Service Graph parameter is supported on versions of MSO/NDO that are 3.3 or greater.
- The recommended approach is to use M(cisco.mso.ndo_schema_template_contract_service_chain) rather than the Contract Service Graph.
- Service Chaining is supported only on ND v3.1 (NDO v4.2.3) and later versions.

author:
- Akini Ross (@akinross)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  contract:
    description:
    - The name of the contract.
    type: str
    required: true
  service_graph:
    description:
    - The service graph to associate with this contract.
    type: str
  service_graph_template:
    description:
    - The template name in which the service graph is located.
    type: str
  service_graph_schema:
    description:
    - The schema name in which the service graph is located.
    type: str
  service_nodes:
    description:
    - A list of nodes and their connector details associated with the Service Graph.
    - The order of the list matches the node id ordering in GUI, so first entry in list will be match node 1.
    type: list
    elements: dict
    suboptions:
      provider:
        description:
        - The name of the Bridge Domain.
        required: true
        type: str
      consumer:
        description:
        - The name of the Bridge Domain.
        required: true
        type: str
      connector_object_type:
        description:
        - The connector ACI object type of the node.
        type: str
        default: bd
        choices: [ bd ]
      provider_schema:
        description:
        - The schema name in which the provider is located.
        type: str
      provider_template:
        description:
         - The template name in which the provider is located.
        type: str
      consumer_schema:
        description:
        - The schema name in which the consumer is located.
        type: str
      consumer_template:
        description:
         - The template name in which the consumer is located.
        type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_contract_filter
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new contract service graph
  cisco.mso.mso_schema_template_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    contract: Contract 1
    service_graph: SG1
    service_graph_nodes:
      - provider: b1
        consumer: b2
    filter: Filter 1
    state: present

- name: Remove a contract service graph
  cisco.mso.mso_schema_template_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    contract: Contract 1
    service_graph: SG1
    state: absent

- name: Query a contract service graph
  cisco.mso.mso_schema_template_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    contract: Contract 1
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_service_graph_connector_spec
from ansible_collections.cisco.mso.plugins.module_utils.constants import SERVICE_NODE_CONNECTOR_MAP


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        contract=dict(type="str", required=True),
        service_graph=dict(type="str"),
        service_graph_template=dict(type="str"),
        service_graph_schema=dict(type="str"),
        service_nodes=dict(type="list", elements="dict", options=mso_service_graph_connector_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["service_graph"]],
            ["state", "present", ["service_graph", "service_nodes"]],
        ],
    )

    schema = module.params.get("schema")
    template_name = module.params.get("template").replace(" ", "")
    contract_name = module.params.get("contract")
    service_graph_name = module.params.get("service_graph")
    service_graph_template = module.params.get("service_graph_template")
    service_graph_schema = module.params.get("service_graph_schema")
    service_nodes = module.params.get("service_nodes")

    state = module.params.get("state")

    mso = MSOModule(module)

    # Initialize variables
    ops = []
    service_graph_obj = None

    # Set path defaults for create logic, if object (contract or filter) is found replace the "-" for specific value
    base_contract_path = "/templates/{0}/contracts".format(template_name)
    service_graph_path = "{0}/{1}/serviceGraphRelationship".format(base_contract_path, contract_name)

    # Get schema
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get template
    template_obj = next((item for item in schema_obj.get("templates") if item.get("name") == template_name), None)
    if not template_obj:
        mso.fail_json(
            msg="Provided template '{0}' does not exist. Existing templates: {1}".format(
                template_name, ", ".join([t.get("name") for t in schema_obj.get("templates")])
            )
        )

    # Get contract
    contract_obj = next((item for item in template_obj.get("contracts") if item.get("name") == contract_name), None)
    if contract_obj:
        # Get service graph if it exists in contract
        if contract_obj.get("serviceGraphRelationship"):
            service_graph_obj = contract_obj.get("serviceGraphRelationship")
            mso.update_service_graph_obj(service_graph_obj)
            mso.existing = service_graph_obj
    else:
        mso.fail_json(
            msg="Provided contract '{0}' does not exist. Existing contracts: {1}".format(
                contract_name, ", ".join([c.get("name") for c in template_obj.get("contracts")])
            )
        )

    if state == "query":
        mso.exit_json()

    mso.previous = mso.existing

    if state == "absent":
        if contract_obj.get("serviceGraphRelationship"):
            mso.existing = {}
            ops.append(dict(op="remove", path=service_graph_path))

    elif state == "present":
        service_nodes_relationship = []
        service_graph_template = service_graph_template.replace(" ", "") if service_graph_template else template_name
        service_graph_schema = service_graph_schema if service_graph_schema else schema
        service_graph_schema_id, service_graph_schema_path, service_graph_schema_obj = mso.query_schema(service_graph_schema)

        # Validation to check if amount of service graph nodes provided is matching the service graph template.
        # The API allows providing more or less service graph nodes behaviour but the GUI does not.
        service_graph_template_obj = next((item for item in service_graph_schema_obj.get("templates") if item.get("name") == service_graph_template), None)
        if not service_graph_template_obj:
            mso.fail_json(
                msg="Provided template '{0}' does not exist. Existing templates: {1}".format(
                    template_name, ", ".join([t.get("name") for t in service_graph_schema_obj.get("templates")])
                )
            )
        service_graph_schema_obj = next((item for item in service_graph_template_obj.get("serviceGraphs") if item.get("name") == service_graph_name), None)
        if service_graph_schema_obj:
            if len(service_nodes) < len(service_graph_schema_obj.get("serviceNodes")):
                mso.fail_json(
                    msg="Not enough service nodes defined, {0} service node(s) provided when {1} needed.".format(
                        len(service_nodes), len(service_graph_schema_obj.get("serviceNodes"))
                    )
                )
            elif len(service_nodes) > len(service_graph_schema_obj.get("serviceNodes")):
                mso.fail_json(
                    msg="Too many service nodes defined, {0} service nodes provided when {1} needed.".format(
                        len(service_nodes), len(service_graph_schema_obj.get("serviceNodes"))
                    )
                )
        else:
            mso.fail_json(msg="Provided service graph '{0}' does not exist.".format(service_graph_name))

        for node_id, service_node in enumerate(service_nodes, 0):
            # Consumer and provider share connector details (so provider/consumer could have separate details in future)
            connector_details = SERVICE_NODE_CONNECTOR_MAP.get(service_node.get("connector_object_type"))
            provider_schema = mso.lookup_schema(service_node.get("provider_schema")) if service_node.get("provider_schema") else schema_id
            provider_template = service_node.get("provider_template").replace(" ", "") if service_node.get("provider_template") else template_name
            consumer_schema = mso.lookup_schema(service_node.get("consumer_schema")) if service_node.get("consumer_schema") else schema_id
            consumer_template = service_node.get("consumer_template").replace(" ", "") if service_node.get("consumer_template") else template_name

            service_nodes_relationship.append(
                {
                    "serviceNodeRef": dict(
                        schemaId=service_graph_schema_id,
                        templateName=service_graph_template,
                        serviceGraphName=service_graph_name,
                        serviceNodeName=service_graph_schema_obj.get("serviceNodes")[node_id].get("name"),
                    ),
                    "providerConnector": {
                        "connectorType": connector_details.get("connector_type"),
                        "{0}Ref".format(connector_details.get("id")): {
                            "schemaId": provider_schema,
                            "templateName": provider_template,
                            "{0}Name".format(connector_details.get("id")): service_node.get("provider"),
                        },
                    },
                    "consumerConnector": {
                        "connectorType": connector_details.get("connector_type"),
                        "{0}Ref".format(connector_details.get("id")): {
                            "schemaId": consumer_schema,
                            "templateName": consumer_template,
                            "{0}Name".format(connector_details.get("id")): service_node.get("consumer"),
                        },
                    },
                }
            )

        service_graph_payload = dict(
            serviceGraphRef=dict(serviceGraphName=service_graph_name, templateName=service_graph_template, schemaId=service_graph_schema_id),
            serviceNodesRelationship=service_nodes_relationship,
        )

        # If service graph exist the operation should be set to "replace" else operation is "add" to create new
        if service_graph_obj:
            ops.append(dict(op="replace", path=service_graph_path, value=service_graph_payload))
        else:
            ops.append(dict(op="add", path=service_graph_path, value=service_graph_payload))

        mso.existing = mso.sent = service_graph_payload

    if not module.check_mode and mso.existing != mso.previous:
        mso.request(schema_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: autoscaling_launch_config_info
version_added: 1.0.0
short_description: Gather information about AWS Autoscaling Launch Configurations
description:
  - Gather information about AWS Autoscaling Launch Configurations.
  - Prior to release 5.0.0 this module was called C(community.aws.ec2_lc_info).
    The usage did not change.
author:
  - "Loïc Latreille (@psykotox)"
options:
  name:
    description:
      - A name or a list of name to match.
    default: []
    type: list
    elements: str
  sort:
    description:
      - Optional attribute which with to sort the results.
    choices: ['launch_configuration_name', 'image_id', 'created_time', 'instance_type', 'kernel_id', 'ramdisk_id', 'key_name']
    type: str
  sort_order:
    description:
      - Order in which to sort results.
      - Only used when the 'sort' parameter is specified.
    choices: ['ascending', 'descending']
    default: 'ascending'
    type: str
  sort_start:
    description:
      - Which result to start with (when sorting).
      - Corresponds to Python slice notation.
    type: int
  sort_end:
    description:
      - Which result to end with (when sorting).
      - Corresponds to Python slice notation.
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all launch configurations
  community.aws.autoscaling_launch_config_info:

- name: Gather information about launch configuration with name "example"
  community.aws.autoscaling_launch_config_info:
    name: example

- name: Gather information sorted by created_time from most recent to least recent
  community.aws.autoscaling_launch_config_info:
    sort: created_time
    sort_order: descending
"""

RETURN = r"""
block_device_mapping:
    description: Block device mapping for the instances of launch configuration.
    type: list
    returned: always
    sample: "[{
        'device_name': '/dev/xvda':,
        'ebs': {
            'delete_on_termination': true,
            'volume_size': 8,
            'volume_type': 'gp2'
    }]"
classic_link_vpc_security_groups:
    description: IDs of one or more security groups for the VPC specified in classic_link_vpc_id.
    type: str
    returned: always
    sample:
created_time:
    description: The creation date and time for the launch configuration.
    type: str
    returned: always
    sample: "2016-05-27T13:47:44.216000+00:00"
ebs_optimized:
    description: EBS I/O optimized C(true) or not C(false).
    type: bool
    returned: always
    sample: true,
image_id:
    description: ID of the Amazon Machine Image (AMI).
    type: str
    returned: always
    sample: "ami-12345678"
instance_monitoring:
    description: Launched with detailed monitoring or not.
    type: dict
    returned: always
    sample: "{
        'enabled': true
    }"
instance_type:
    description: Instance type.
    type: str
    returned: always
    sample: "t2.micro"
kernel_id:
    description: ID of the kernel associated with the AMI.
    type: str
    returned: always
    sample:
key_name:
    description: Name of the key pair.
    type: str
    returned: always
    sample: "user_app"
launch_configuration_arn:
    description: Amazon Resource Name (ARN) of the launch configuration.
    type: str
    returned: always
    sample: "arn:aws:autoscaling:us-east-1:123456798012:launchConfiguration:ba785e3a-dd42-6f02-4585-ea1a2b458b3d:launchConfigurationName/lc-app"
launch_configuration_name:
    description: Name of the launch configuration.
    type: str
    returned: always
    sample: "lc-app"
ramdisk_id:
    description: ID of the RAM disk associated with the AMI.
    type: str
    returned: always
    sample:
security_groups:
    description: Security groups to associated.
    type: list
    returned: always
    sample: "[
        'web'
    ]"
user_data:
    description: User data available.
    type: str
    returned: always
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def list_launch_configs(connection, module):
    launch_config_name = module.params.get("name")
    sort = module.params.get("sort")
    sort_order = module.params.get("sort_order")
    sort_start = module.params.get("sort_start")
    sort_end = module.params.get("sort_end")

    try:
        pg = connection.get_paginator("describe_launch_configurations")
        launch_configs = pg.paginate(LaunchConfigurationNames=launch_config_name).build_full_result()
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e, msg="Failed to list launch configs")

    snaked_launch_configs = []
    for launch_config in launch_configs["LaunchConfigurations"]:
        snaked_launch_configs.append(camel_dict_to_snake_dict(launch_config))

    for launch_config in snaked_launch_configs:
        if "CreatedTime" in launch_config:
            launch_config["CreatedTime"] = str(launch_config["CreatedTime"])

    if sort:
        snaked_launch_configs.sort(key=lambda e: e[sort], reverse=(sort_order == "descending"))

    if sort and sort_start and sort_end:
        snaked_launch_configs = snaked_launch_configs[sort_start:sort_end]
    elif sort and sort_start:
        snaked_launch_configs = snaked_launch_configs[sort_start:]
    elif sort and sort_end:
        snaked_launch_configs = snaked_launch_configs[:sort_end]

    module.exit_json(launch_configurations=snaked_launch_configs)


def main():
    argument_spec = dict(
        name=dict(required=False, default=[], type="list", elements="str"),
        sort=dict(
            required=False,
            default=None,
            choices=[
                "launch_configuration_name",
                "image_id",
                "created_time",
                "instance_type",
                "kernel_id",
                "ramdisk_id",
                "key_name",
            ],
        ),
        sort_order=dict(required=False, default="ascending", choices=["ascending", "descending"]),
        sort_start=dict(required=False, type="int"),
        sort_end=dict(required=False, type="int"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        connection = module.client("autoscaling")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    list_launch_configs(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_test_sic_status
short_description: Test SIC Status reflects the state of the gateway after it has received the certificate issued by the
                   ICA. If the SIC status is Unknown then there is no connection between the gateway and the Security
                   Management Server. If the SIC status is No Communication, an error message will appear. It may
                   contain specific instructions on how to fix the situation.
description:
  - Test SIC Status reflects the state of the gateway after it has received the certificate issued by the ICA. If the SIC status is Unknown then there is
    no connection between the gateway and the Security Management Server. If the SIC status is No Communication, an error message will appear. It may contain
    specific instructions on how to fix the situation.
  - All operations are performed over Web Services API.
  - Available from R81.20 management version.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Gateway, cluster member or Check Point host name.
    type: str
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: test-sic-status
  cp_mgmt_test_sic_status:
    name: gw1
"""

RETURN = """
cp_mgmt_test_sic_status:
  description: The checkpoint test-sic-status output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(name=dict(type="str"))
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "test-sic-status"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator, )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
    meraki_compare_equality2,
    get_dict_result,
)
from ansible_collections.cisco.meraki.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = meraki_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    name=dict(type="str"),
    radiusServers=dict(type="list"),
    radius=dict(type="dict"),
    guestPortBouncing=dict(type="bool"),
    radiusTestingEnabled=dict(type="bool"),
    radiusCoaSupportEnabled=dict(type="bool"),
    radiusAccountingEnabled=dict(type="bool"),
    radiusAccountingServers=dict(type="list"),
    radiusGroupAttribute=dict(type="str"),
    hostMode=dict(type="str"),
    accessPolicyType=dict(type="str"),
    increaseAccessSpeed=dict(type="bool"),
    guestVlanId=dict(type="int"),
    dot1x=dict(type="dict"),
    voiceVlanClients=dict(type="bool"),
    urlRedirectWalledGardenEnabled=dict(type="bool"),
    urlRedirectWalledGardenRanges=dict(type="list"),
    guestGroupPolicyId=dict(type="str"),
    guestSgtId=dict(type="int"),
    networkId=dict(type="str"),
    accessPolicyNumber=dict(type="str"),
))

required_if = [
    ("state", "present", ["accessPolicyNumber", "name", "networkId"], True),
    ("state", "absent", ["accessPolicyNumber", "name", "networkId"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class NetworksSwitchAccessPolicies(object):
    def __init__(self, params, meraki):
        self.meraki = meraki
        self.new_object = dict(
            name=params.get("name"),
            radiusServers=params.get("radiusServers"),
            radius=params.get("radius"),
            guestPortBouncing=params.get("guestPortBouncing"),
            radiusTestingEnabled=params.get("radiusTestingEnabled"),
            radiusCoaSupportEnabled=params.get("radiusCoaSupportEnabled"),
            radiusAccountingEnabled=params.get("radiusAccountingEnabled"),
            radiusAccountingServers=params.get("radiusAccountingServers"),
            radiusGroupAttribute=params.get("radiusGroupAttribute"),
            hostMode=params.get("hostMode"),
            accessPolicyType=params.get("accessPolicyType"),
            increaseAccessSpeed=params.get("increaseAccessSpeed"),
            guestVlanId=params.get("guestVlanId"),
            dot1x=params.get("dot1x"),
            voiceVlanClients=params.get("voiceVlanClients"),
            urlRedirectWalledGardenEnabled=params.get("urlRedirectWalledGardenEnabled"),
            urlRedirectWalledGardenRanges=params.get("urlRedirectWalledGardenRanges"),
            guestGroupPolicyId=params.get("guestGroupPolicyId"),
            guestSgtId=params.get("guestSgtId"),
            networkId=params.get("networkId"),
            accessPolicyNumber=params.get("accessPolicyNumber"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        return new_object_params

    def get_params_by_id(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('accessPolicyNumber') is not None or self.new_object.get(
                'access_policy_number') is not None:
            new_object_params['accessPolicyNumber'] = self.new_object.get(
                'accessPolicyNumber') or self.new_object.get('access_policy_number')
        return new_object_params

    def create_params(self):
        new_object_params = {}
        if self.new_object.get('name') is not None or self.new_object.get(
                'name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('radiusServers') is not None or self.new_object.get(
                'radius_servers') is not None:
            new_object_params['radiusServers'] = self.new_object.get(
                'radiusServers') or self.new_object.get('radius_servers')
        if self.new_object.get('radius') is not None or self.new_object.get(
                'radius') is not None:
            new_object_params['radius'] = self.new_object.get('radius') or \
                self.new_object.get('radius')
        if self.new_object.get('guestPortBouncing') is not None or self.new_object.get(
                'guest_port_bouncing') is not None:
            new_object_params['guestPortBouncing'] = self.new_object.get(
                'guestPortBouncing')
        if self.new_object.get('radiusTestingEnabled') is not None or self.new_object.get(
                'radius_testing_enabled') is not None:
            new_object_params['radiusTestingEnabled'] = self.new_object.get(
                'radiusTestingEnabled')
        if self.new_object.get('radiusCoaSupportEnabled') is not None or self.new_object.get(
                'radius_coa_support_enabled') is not None:
            new_object_params['radiusCoaSupportEnabled'] = self.new_object.get(
                'radiusCoaSupportEnabled')
        if self.new_object.get('radiusAccountingEnabled') is not None or self.new_object.get(
                'radius_accounting_enabled') is not None:
            new_object_params['radiusAccountingEnabled'] = self.new_object.get(
                'radiusAccountingEnabled')
        if self.new_object.get('radiusAccountingServers') is not None or self.new_object.get(
                'radius_accounting_servers') is not None:
            new_object_params['radiusAccountingServers'] = self.new_object.get(
                'radiusAccountingServers') or self.new_object.get('radius_accounting_servers')
        if self.new_object.get('radiusGroupAttribute') is not None or self.new_object.get(
                'radius_group_attribute') is not None:
            new_object_params['radiusGroupAttribute'] = self.new_object.get(
                'radiusGroupAttribute') or self.new_object.get('radius_group_attribute')
        if self.new_object.get('hostMode') is not None or self.new_object.get(
                'host_mode') is not None:
            new_object_params['hostMode'] = self.new_object.get(
                'hostMode') or self.new_object.get('host_mode')
        if self.new_object.get('accessPolicyType') is not None or self.new_object.get(
                'access_policy_type') is not None:
            new_object_params['accessPolicyType'] = self.new_object.get(
                'accessPolicyType') or self.new_object.get('access_policy_type')
        if self.new_object.get('increaseAccessSpeed') is not None or self.new_object.get(
                'increase_access_speed') is not None:
            new_object_params['increaseAccessSpeed'] = self.new_object.get(
                'increaseAccessSpeed')
        if self.new_object.get('guestVlanId') is not None or self.new_object.get(
                'guest_vlan_id') is not None:
            new_object_params['guestVlanId'] = self.new_object.get(
                'guestVlanId') or self.new_object.get('guest_vlan_id')
        if self.new_object.get('dot1x') is not None or self.new_object.get(
                'dot1x') is not None:
            new_object_params['dot1x'] = self.new_object.get('dot1x') or \
                self.new_object.get('dot1x')
        if self.new_object.get('voiceVlanClients') is not None or self.new_object.get(
                'voice_vlan_clients') is not None:
            new_object_params['voiceVlanClients'] = self.new_object.get(
                'voiceVlanClients')
        if self.new_object.get('urlRedirectWalledGardenEnabled') is not None or self.new_object.get(
                'url_redirect_walled_garden_enabled') is not None:
            new_object_params['urlRedirectWalledGardenEnabled'] = self.new_object.get(
                'urlRedirectWalledGardenEnabled')
        if self.new_object.get('urlRedirectWalledGardenRanges') is not None or self.new_object.get(
                'url_redirect_walled_garden_ranges') is not None:
            new_object_params['urlRedirectWalledGardenRanges'] = self.new_object.get(
                'urlRedirectWalledGardenRanges') or self.new_object.get('url_redirect_walled_garden_ranges')
        if self.new_object.get('guestGroupPolicyId') is not None or self.new_object.get(
                'guest_group_policy_id') is not None:
            new_object_params['guestGroupPolicyId'] = self.new_object.get(
                'guestGroupPolicyId') or self.new_object.get('guest_group_policy_id')
        if self.new_object.get('guestSgtId') is not None or self.new_object.get(
                'guest_sgt_id') is not None:
            new_object_params['guestSgtId'] = self.new_object.get(
                'guestSgtId') or self.new_object.get('guest_sgt_id')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        return new_object_params

    def delete_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('accessPolicyNumber') is not None or self.new_object.get(
                'access_policy_number') is not None:
            new_object_params['accessPolicyNumber'] = self.new_object.get(
                'accessPolicyNumber') or self.new_object.get('access_policy_number')
        return new_object_params

    def update_by_id_params(self):
        new_object_params = {}
        if self.new_object.get('name') is not None or self.new_object.get(
                'name') is not None:
            new_object_params['name'] = self.new_object.get('name') or \
                self.new_object.get('name')
        if self.new_object.get('radiusServers') is not None or self.new_object.get(
                'radius_servers') is not None:
            new_object_params['radiusServers'] = self.new_object.get(
                'radiusServers') or self.new_object.get('radius_servers')
        if self.new_object.get('radius') is not None or self.new_object.get(
                'radius') is not None:
            new_object_params['radius'] = self.new_object.get('radius') or \
                self.new_object.get('radius')
        if self.new_object.get('guestPortBouncing') is not None or self.new_object.get(
                'guest_port_bouncing') is not None:
            new_object_params['guestPortBouncing'] = self.new_object.get(
                'guestPortBouncing')
        if self.new_object.get('radiusTestingEnabled') is not None or self.new_object.get(
                'radius_testing_enabled') is not None:
            new_object_params['radiusTestingEnabled'] = self.new_object.get(
                'radiusTestingEnabled')
        if self.new_object.get('radiusCoaSupportEnabled') is not None or self.new_object.get(
                'radius_coa_support_enabled') is not None:
            new_object_params['radiusCoaSupportEnabled'] = self.new_object.get(
                'radiusCoaSupportEnabled')
        if self.new_object.get('radiusAccountingEnabled') is not None or self.new_object.get(
                'radius_accounting_enabled') is not None:
            new_object_params['radiusAccountingEnabled'] = self.new_object.get(
                'radiusAccountingEnabled')
        if self.new_object.get('radiusAccountingServers') is not None or self.new_object.get(
                'radius_accounting_servers') is not None:
            new_object_params['radiusAccountingServers'] = self.new_object.get(
                'radiusAccountingServers') or self.new_object.get('radius_accounting_servers')
        if self.new_object.get('radiusGroupAttribute') is not None or self.new_object.get(
                'radius_group_attribute') is not None:
            new_object_params['radiusGroupAttribute'] = self.new_object.get(
                'radiusGroupAttribute') or self.new_object.get('radius_group_attribute')
        if self.new_object.get('hostMode') is not None or self.new_object.get(
                'host_mode') is not None:
            new_object_params['hostMode'] = self.new_object.get(
                'hostMode') or self.new_object.get('host_mode')
        if self.new_object.get('accessPolicyType') is not None or self.new_object.get(
                'access_policy_type') is not None:
            new_object_params['accessPolicyType'] = self.new_object.get(
                'accessPolicyType') or self.new_object.get('access_policy_type')
        if self.new_object.get('increaseAccessSpeed') is not None or self.new_object.get(
                'increase_access_speed') is not None:
            new_object_params['increaseAccessSpeed'] = self.new_object.get(
                'increaseAccessSpeed')
        if self.new_object.get('guestVlanId') is not None or self.new_object.get(
                'guest_vlan_id') is not None:
            new_object_params['guestVlanId'] = self.new_object.get(
                'guestVlanId') or self.new_object.get('guest_vlan_id')
        if self.new_object.get('dot1x') is not None or self.new_object.get(
                'dot1x') is not None:
            new_object_params['dot1x'] = self.new_object.get('dot1x') or \
                self.new_object.get('dot1x')
        if self.new_object.get('voiceVlanClients') is not None or self.new_object.get(
                'voice_vlan_clients') is not None:
            new_object_params['voiceVlanClients'] = self.new_object.get(
                'voiceVlanClients')
        if self.new_object.get('urlRedirectWalledGardenEnabled') is not None or self.new_object.get(
                'url_redirect_walled_garden_enabled') is not None:
            new_object_params['urlRedirectWalledGardenEnabled'] = self.new_object.get(
                'urlRedirectWalledGardenEnabled')
        if self.new_object.get('urlRedirectWalledGardenRanges') is not None or self.new_object.get(
                'url_redirect_walled_garden_ranges') is not None:
            new_object_params['urlRedirectWalledGardenRanges'] = self.new_object.get(
                'urlRedirectWalledGardenRanges') or self.new_object.get('url_redirect_walled_garden_ranges')
        if self.new_object.get('guestGroupPolicyId') is not None or self.new_object.get(
                'guest_group_policy_id') is not None:
            new_object_params['guestGroupPolicyId'] = self.new_object.get(
                'guestGroupPolicyId') or self.new_object.get('guest_group_policy_id')
        if self.new_object.get('guestSgtId') is not None or self.new_object.get(
                'guest_sgt_id') is not None:
            new_object_params['guestSgtId'] = self.new_object.get(
                'guestSgtId') or self.new_object.get('guest_sgt_id')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('accessPolicyNumber') is not None or self.new_object.get(
                'access_policy_number') is not None:
            new_object_params['accessPolicyNumber'] = self.new_object.get(
                'accessPolicyNumber') or self.new_object.get('access_policy_number')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method or it is in another action
        try:
            items = self.meraki.exec_meraki(
                family="switch",
                function="getNetworkSwitchAccessPolicies",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
            if result is None:
                result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        try:
            items = self.meraki.exec_meraki(
                family="switch",
                function="getNetworkSwitchAccessPolicy",
                params=self.get_params_by_id()
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'accessPolicyNumber', id)
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def exists(self):
        id_exists = False
        name_exists = False
        prev_obj = None
        o_id = self.new_object.get("id")
        o_id = o_id or self.new_object.get(
            "access_policy_number") or self.new_object.get("accessPolicyNumber")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            _id = _id or prev_obj.get("accessPolicyNumber")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
                self.new_object.update(dict(accessPolicyNumber=_id))
            if _id:
                prev_obj = self.get_object_by_id(_id)
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("name", "name"),
            ("radiusServers", "radiusServers"),
            ("radius", "radius"),
            ("guestPortBouncing", "guestPortBouncing"),
            ("radiusTestingEnabled", "radiusTestingEnabled"),
            ("radiusCoaSupportEnabled", "radiusCoaSupportEnabled"),
            ("radiusAccountingEnabled", "radiusAccountingEnabled"),
            ("radiusAccountingServers", "radiusAccountingServers"),
            ("radiusGroupAttribute", "radiusGroupAttribute"),
            ("hostMode", "hostMode"),
            ("accessPolicyType", "accessPolicyType"),
            ("increaseAccessSpeed", "increaseAccessSpeed"),
            ("guestVlanId", "guestVlanId"),
            ("dot1x", "dot1x"),
            ("voiceVlanClients", "voiceVlanClients"),
            ("urlRedirectWalledGardenEnabled", "urlRedirectWalledGardenEnabled"),
            ("urlRedirectWalledGardenRanges", "urlRedirectWalledGardenRanges"),
            ("guestGroupPolicyId", "guestGroupPolicyId"),
            ("guestSgtId", "guestSgtId"), ("accessPolicyNumber", "accessPolicyNumber"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (DNAC) params
        # If any does not have eq params, it requires update
        return any(
            not meraki_compare_equality2(
                current_obj.get(meraki_param),
                requested_obj.get(ansible_param)) for (
                meraki_param,
                ansible_param) in obj_params)

    def create(self):
        result = self.meraki.exec_meraki(
            family="switch",
            function="createNetworkSwitchAccessPolicy",
            params=self.create_params(),
            op_modifies=True,
        )
        return result

    def update(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("accessPolicyNumber")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("accessPolicyNumber")
            if id_:
                self.new_object.update(dict(accessPolicyNumber=id_))
        result = self.meraki.exec_meraki(
            family="switch",
            function="updateNetworkSwitchAccessPolicy",
            params=self.update_by_id_params(),
            op_modifies=True,
        )
        return result

    def delete(self):
        id = self.new_object.get("id")
        id = id or self.new_object.get("accessPolicyNumber")
        name = self.new_object.get("name")
        result = None
        if not id:
            prev_obj_name = self.get_object_by_name(name)
            id_ = None
            if prev_obj_name:
                id_ = prev_obj_name.get("id")
                id_ = id_ or prev_obj_name.get("accessPolicyNumber")
            if id_:
                self.new_object.update(dict(accessPolicyNumber=id_))
        result = self.meraki.exec_meraki(
            family="switch",
            function="deleteNetworkSwitchAccessPolicy",
            params=self.delete_by_id_params(),
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        meraki = MERAKI(self._task.args)
        obj = NetworksSwitchAccessPolicies(self._task.args, meraki)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    meraki.object_updated()
                else:
                    response = prev_obj
                    meraki.object_already_present()
            else:
                response = obj.create()
                meraki.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                response = obj.delete()
                meraki.object_deleted()
            else:
                meraki.object_already_absent()

        self._result.update(dict(meraki_response=response))
        self._result.update(meraki.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_find
short_description: Return a list of files based on specific criteria
description:
    - Return a list of files based on specified criteria.
    - Multiple criteria are AND'd together.
    - For non-Windows targets, use the M(ansible.builtin.find) module instead.
options:
    age:
        description:
            - Select files or folders whose age is equal to or greater than
              the specified time.
            - Use a negative age to find files equal to or less than
              the specified time.
            - You can choose seconds, minutes, hours, days or weeks
              by specifying the first letter of an of
              those words (for example, "2s", "10d", 1w").
        type: str
    age_stamp:
        description:
            - Choose the file property against which we compare C(age).
            - The default attribute we compare with is the last modification time.
        type: str
        choices: [ atime, ctime, mtime ]
        default: mtime
    checksum_algorithm:
        description:
            - Algorithm to determine the checksum of a file.
            - Will throw an error if the host is unable to use specified algorithm.
        type: str
        choices: [ md5, sha1, sha256, sha384, sha512 ]
        default: sha1
    depth:
        description:
            - Set the maximum number of levels to descend into.
            - Setting recurse to C(false) will override this value, which is effectively depth 1.
            - Default depth is unlimited.
        type: int
        version_added: 2.0.0
    file_type:
        description:
            - Type of file to search for.
            - Specify V(any) to find both file and directory. Added in version 3.2.0.
        type: str
        choices: [ directory, file, any ]
        default: file
    follow:
        description:
            - Set this to C(true) to follow symlinks in the path.
            - This needs to be used in conjunction with C(recurse).
        type: bool
        default: no
    get_checksum:
        description:
            - Whether to return a checksum of the file in the return info (default sha1),
              use C(checksum_algorithm) to change from the default.
        type: bool
        default: yes
    hidden:
        description: Set this to include hidden files or folders.
        type: bool
        default: no
    paths:
        description:
            - List of paths of directories to search for files or folders in.
            - This can be supplied as a single path or a list of paths.
        type: list
        elements: str
        required: yes
    patterns:
        description:
            - One or more (powershell or regex) patterns to compare filenames with.
            - The type of pattern matching is controlled by C(use_regex) option.
            - The patterns restrict the list of files or folders to be returned based on the filenames.
            - For a file to be matched it only has to match with one pattern in a list provided.
        type: list
        elements: str
        aliases: [ "regex", "regexp" ]
    recurse:
        description:
            - Will recursively descend into the directory looking for files or folders.
        type: bool
        default: no
    size:
        description:
            - Select files or folders whose size is equal to or greater than the specified size.
            - Use a negative value to find files equal to or less than the specified size.
            - You can specify the size with a suffix of the byte type i.e. kilo = k, mega = m...
            - Size is not evaluated for symbolic links.
        type: str
    use_regex:
        description:
            - Will set patterns to run as a regex check if set to C(true).
        type: bool
        default: no
    case_sensitive:
        description:
            - Make case-sensitive string comparisons for the filename match.
            - Set C(use_regex) to V(true) to make this work.
        type: bool
        default: no
        version_added: 3.0.0
notes:
- When scanning directories with a large number of files containing lots of data it is recommended to set
  C(get_checksum=false). This will speed up the time it takes to scan the folders as getting a checksum needs to
  read the contents of every file it returns.
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: Find files in path
  ansible.windows.win_find:
    paths: D:\Temp

- name: Find hidden files in path
  ansible.windows.win_find:
    paths: D:\Temp
    hidden: true

- name: Find files in multiple paths
  ansible.windows.win_find:
    paths:
      - C:\Temp
      - D:\Temp

- name: Find files in directory while searching recursively
  ansible.windows.win_find:
    paths: D:\Temp
    recurse: true

- name: Find files in directory while following symlinks
  ansible.windows.win_find:
    paths: D:\Temp
    recurse: true
    follow: true

- name: Find files with .log and .out extension using powershell wildcards
  ansible.windows.win_find:
    paths: D:\Temp
    patterns: ['*.log', '*.out']

- name: Find files in path based on regex pattern
  ansible.windows.win_find:
    paths: D:\Temp
    patterns: out_\d{8}-\d{6}.log

- name: Find files older than 1 day
  ansible.windows.win_find:
    paths: D:\Temp
    age: 86400

- name: Find files older than 1 day based on create time
  ansible.windows.win_find:
    paths: D:\Temp
    age: 86400
    age_stamp: ctime

- name: Find files older than 1 day with unit syntax
  ansible.windows.win_find:
    paths: D:\Temp
    age: 1d

- name: Find files newer than 1 hour
  ansible.windows.win_find:
    paths: D:\Temp
    age: -3600

- name: Find files newer than 1 hour with unit syntax
  ansible.windows.win_find:
    paths: D:\Temp
    age: -1h

- name: Find files larger than 1MB
  ansible.windows.win_find:
    paths: D:\Temp
    size: 1048576

- name: Find files larger than 1GB with unit syntax
  ansible.windows.win_find:
    paths: D:\Temp
    size: 1g

- name: Find files smaller than 1MB
  ansible.windows.win_find:
    paths: D:\Temp
    size: -1048576

- name: Find files smaller than 1GB with unit syntax
  ansible.windows.win_find:
    paths: D:\Temp
    size: -1g

- name: Find folders/symlinks in multiple paths
  ansible.windows.win_find:
    paths:
      - C:\Temp
      - D:\Temp
    file_type: directory

- name: Find files and return SHA256 checksum of files found
  ansible.windows.win_find:
    paths: C:\Temp
    get_checksum: true
    checksum_algorithm: sha256

- name: Find files and do not return the checksum
  ansible.windows.win_find:
    paths: C:\Temp
    get_checksum: false
'''

RETURN = r'''
examined:
    description: The number of files/folders that was checked.
    returned: always
    type: int
    sample: 10
matched:
    description: The number of files/folders that match the criteria.
    returned: always
    type: int
    sample: 2
files:
    description: Information on the files/folders that match the criteria returned as a list of dictionary elements
      for each file matched. The entries are sorted by the path value alphabetically.
    returned: success
    type: complex
    contains:
        attributes:
            description: attributes of the file at path in raw form.
            returned: success, path exists
            type: str
            sample: "Archive, Hidden"
        checksum:
            description: The checksum of a file based on checksum_algorithm specified.
            returned: success, path exists, path is a file, get_checksum == True
            type: str
            sample: 09cb79e8fc7453c84a07f644e441fd81623b7f98
        creationtime:
            description: The create time of the file represented in seconds since epoch.
            returned: success, path exists
            type: float
            sample: 1477984205.15
        exists:
            description: Whether the file exists, will always be true for M(ansible.windows.win_find).
            returned: success, path exists
            type: bool
            sample: true
        extension:
            description: The extension of the file at path.
            returned: success, path exists, path is a file
            type: str
            sample: ".ps1"
        filename:
            description: The name of the file.
            returned: success, path exists
            type: str
            sample: temp
        hlnk_targets:
            description: List of other files pointing to the same file (hard links), excludes the current file.
            returned: success, path exists
            type: list
            sample:
            - C:\temp\file.txt
            - C:\Windows\update.log
        isarchive:
            description: If the path is ready for archiving or not.
            returned: success, path exists
            type: bool
            sample: true
        isdir:
            description: If the path is a directory or not.
            returned: success, path exists
            type: bool
            sample: true
        ishidden:
            description: If the path is hidden or not.
            returned: success, path exists
            type: bool
            sample: true
        isjunction:
            description: If the path is a junction point.
            returned: success, path exists
            type: bool
            sample: true
        islnk:
            description: If the path is a symbolic link.
            returned: success, path exists
            type: bool
            sample: true
        isreadonly:
            description: If the path is read only or not.
            returned: success, path exists
            type: bool
            sample: true
        isreg:
            description: If the path is a regular file or not.
            returned: success, path exists
            type: bool
            sample: true
        isshared:
            description: If the path is shared or not.
            returned: success, path exists
            type: bool
            sample: true
        lastaccesstime:
            description: The last access time of the file represented in seconds since epoch.
            returned: success, path exists
            type: float
            sample: 1477984205.15
        lastwritetime:
            description: The last modification time of the file represented in seconds since epoch.
            returned: success, path exists
            type: float
            sample: 1477984205.15
        lnk_source:
            description: The target of the symlink normalized for the remote filesystem.
            returned: success, path exists, path is a symbolic link or junction point
            type: str
            sample: C:\temp
        lnk_target:
            description: The target of the symlink. Note that relative paths remain relative, will return null if not a link.
            returned: success, path exists, path is a symbolic link or junction point
            type: str
            sample: temp
        nlink:
            description: Number of links to the file (hard links)
            returned: success, path exists
            type: int
            sample: 1
        owner:
            description: The owner of the file.
            returned: success, path exists
            type: str
            sample: BUILTIN\Administrators
        path:
            description: The full absolute path to the file.
            returned: success, path exists
            type: str
            sample: C:\temp\file.txt
        sharename:
            description: The name of share if folder is shared.
            returned: success, path exists, path is a directory and isshared == True
            type: str
            sample: file-share
        size:
            description: The size in bytes of the file.
            returned: success, path exists, path is a file
            type: int
            sample: 1024
'''
